import ConfigParser
import os
from panda3d.core import Point3
from direct.interval.IntervalGlobal import Parallel
from direct.interval.IntervalGlobal import Sequence
from direct.showbase.DirectObject import DirectObject

class Card(DirectObject):
	"""This class represents a card of the game. It contains all it's
	values and the model itself. It also handles loading of the card
	data of given card files"""
	def __init__(self, cardFile):
		"""Load the given card file"""
		config = ConfigParser.RawConfigParser()
		config.read(cardFile)
		self.id = config.getint("card", "id")
		self.name = config.get("card", "name")
		self.release = config.getint("card", "first-release")
		self.active = config.getboolean("card", "active")
		self.support = config.getfloat("card", "support")
		self.texturePath = os.path.join(os.path.dirname(cardFile),
										"{:02d}.png".format(self.id))
		self.texture = loader.loadTexture(self.texturePath)
		# load the card model
		self.model = loader.loadModel("Data/Card.egg")
		self.model.setScale(0.5)
		self.model.setR(180)
		self.model.setH(180)
		# set the card's texture to the one specified for this card
		self.model.setTexture(self.texture, 1)

	def getCardOptions(self):
		"""Get all the options a card will have"""
		return ["First Release", "Status", "Support"]

	def getCardOption(self, option):
		"""Get the value of the given option"""
		if option == "First Release": return self.release
		elif option == "Status": return self.active
		elif option == "Support": return self.support

	def compare(self, option, value):
		"""Compares the given value with the value of the same option
		of this card and returns true if this card is superior to the
		value in the argument, false otherwise"""
		# NOTE: value is always of type string and has to be converted
		#       to the desired type to be correctly comparable
		ret = False
		if option == "First Release":
			# check the release value, the earlier the system was
			# released, the better. On a draw the opponent wins.
			if self.release < int(value):
				ret = True
		elif option == "Status":
			# check the status value, active is better than inactive.
			# On a draw the opponent wins.
			val = False
			if value == "True":
				val = True
			if self.active == True and val == False:
				ret = True
		elif option == "Support":
			# check the support value, the longer a support, the better.
			# On a draw the opponent wins.
			if self.support > float(value):
				ret = True
		return ret

	def drawCard(self):
		"""This method plays an animation to draw this card.
		It will then be placed in the middle of the main screen. The
		return value of this function is the name of an event which will
		be thrown after the draw-animation is done."""
		# move card from deck to table center
		drawCardAnimP1 = self.model.posInterval(0.5, Point3(-0.25, self.model.getY(), self.model.getZ()))
		# turn card and move to screen center
		cardPosInterval = self.model.posInterval(1, Point3(-0.25, 4, 0.25))
		cardHprInterval = self.model.hprInterval(1, Point3(180, 90, 180))
		drawCardAnimP2 = Parallel(
			cardPosInterval,
			cardHprInterval,
			name="drawCardPart2{}".format(self.id))
		# put all animation parts together
		drawCardAnim = Sequence(
			drawCardAnimP1,
			drawCardAnimP2,
			name="drawCardFull{}".format(self.id))
		# now play the animation and set an event name
		drawCardAnim.start()
		drawCardAnim.setDoneEvent("drawCardAnimDone{}".format(self.id))
		return "drawCardAnimDone{}".format(self.id)

	def drawOpCard(self):
		"""This method plays an animation to draw an opponents card.
		It will then be placed in the middle of the main screen at the
		other side of the table. The return value of this function is
		the name of an event which will be thrown after the
		draw-animation is done."""
		# rotate the opponent card by 180 degree to draw show it correctly
		self.model.setH(0)
		# move card from deck to table center
		drawOpCardAnimP1 = self.model.posInterval(0.5, Point3(-0.25, self.model.getY(), self.model.getZ()))
		# turn card and move to screen center
		cardOpPosInterval = self.model.posInterval(1, Point3(-0.25, 10, 0.25))
		cardOpHprInterval = self.model.hprInterval(1, Point3(0, 90, 180))
		drawOpCardAnimP2 = Parallel(
			cardOpPosInterval,
			cardOpHprInterval,
			name="drawOpCardPart2{}".format(self.id))
		# put all animation parts together
		drawOpCardAnim = Sequence(
			drawOpCardAnimP1,
			drawOpCardAnimP2,
			name="drawOpCardFull{}".format(self.id))
		# now play the animation and set an event name
		drawOpCardAnim.start()
		drawOpCardAnim.setDoneEvent("drawOpCardAnimDone{}".format(self.id))
		return "drawOpCardAnimDone{}".format(self.id)

	def hideCard(self):
		"""Play an animation and hide the card"""
		cardScaleInterval = self.model.scaleInterval(1.0, 0.0)
		cardScaleInterval.setDoneEvent("cardHideable{}".format(self.id))
		self.accept(cardScaleInterval.getDoneEvent(), self.model.hide)
		cardScaleInterval.start()

	def showCard(self):
		"""Show a previously hidden card"""
		self.model.setScale(0.5)
		self.model.show()
