import os
from GameGui import Chat
from GameGui import CardSelection
from GameGui import EndGame
from GameGui import LoadingScreen
from Card import Card
from direct.showbase.DirectObject import DirectObject
from panda3d.core import PointLight
from panda3d.core import Spotlight
from panda3d.core import PerspectiveLens
from panda3d.core import AmbientLight
from panda3d.core import VBase4

class Game(DirectObject):
	def __init__(self):
		"""Initialization of the main game class
		set up all variables and the games hud. After this function call
		all events will be catched by this class"""
		# setup the loading screen
		self.ls = LoadingScreen()

		# set the loading value of our loading screen
		base.messenger.send("updateLoadingScreen", [0])

		# setup variables
		self.cardDeck = []
		self.playerDeck = []
		self.opponentDeck = []
		self.loadCards()
		self.currentCard = None
		self.currentCardOp = None
		self.points = 0
		self.opponentPoints = 0
		self.currentCardIdx = 9

		base.messenger.send("updateLoadingScreen", [1])

		# setup GUI stuff
		self.chat = Chat()
		self.cardSelection = CardSelection(self.cardDeck[0].getCardOptions())
		self.cardSelection.hide()

		base.messenger.send("updateLoadingScreen", [15])

		# load the table model
		table = loader.loadModel("Data/Table.egg")
		table.setPos(-1, 10, -1.25)
		table.reparentTo(render)

		base.messenger.send("updateLoadingScreen", [80])

		# set up some nice lighting
        # A pointlight directly above the table
		tablelight = PointLight('tablelight')
		tablelight.setColor(VBase4(.5, .5, .5, 1))
		self.tablelightNP = render.attachNewNode(tablelight)
		self.tablelightNP.setPos(-1, 5, 1)
		render.setLight(self.tablelightNP)

		# basic ambient light
		alight = AmbientLight('ambientlight')
		alight.setColor(VBase4(0.02, 0.02, 0.02, 1))
		self.alNP = render.attachNewNode(alight)
		render.setLight(self.alNP)

        # A pointlight at the "players head"
		playerlight = PointLight('playerlight')
		playerlight.setColor(VBase4(1, 1, 1, 1))
		self.playerlightNP = render.attachNewNode(playerlight)
		self.playerlightNP.setPos(-0.25, 3, 0.5)
		render.setLight(self.playerlightNP)

		base.messenger.send("updateLoadingScreen", [90])

		# send a request to get the players card deck
		base.messenger.send("requestClientDeck")

		base.messenger.send("updateLoadingScreen", [91])

		# accept all events
		self.accept("gotDeck", self.setPlayerDeck)
		self.accept("opponentSelection", self.checkOpponentSelection)
		self.accept("guiSelected", self.selectionMade)
		self.accept("opponentBetter", self.opponentIsBetter)
		self.accept("setTurn", self.setTurn)
		self.accept("endGame", self.endGame)
		self.accept("opponentOnline", self.chat.opponentOnline)

	def setTurn(self, isOurTurn):
		"""set the turn of the player to either True or False. If it's
		the players turn the card selection gui will be shown and hidden
		if it's not the players turn."""
		if isOurTurn:
			# it's our turn, so show the gui
			self.cardSelection.show()
		else:
			# hide the gui as it's not our turn
			self.cardSelection.hide()

	def selectionMade(self, selection):
		"""Send the on the gui selected value and the actual card value
		to the other clients"""
		base.messenger.send(
			"sendMessage",
			["GET_CARD_OPTION:opponentSelection:{}|{}".format(
				selection, self.currentCard.getCardOption(selection))])

	def checkOpponentSelection(self, selection):
		"""This method checks the selection of the opponent and sends
		an event. Dependent on the values of our and the opponents card
		it will send a positive or negative value. This function will
		also send an event to update the points shown on the GUI"""
		# split up the values of the selection
		values = selection.split("|")
		# check if we or the opponent has the better card value
		if self.currentCard.compare(values[0], values[1]):
			# we have the better card
			base.messenger.send(
				"sendMessage",
				["GET_CARD_OPTION:opponentBetter:True"])
			self.points += 1
		else:
			# the opponent has the better card
			base.messenger.send(
				"sendMessage",
				["GET_CARD_OPTION:opponentBetter:False"])
			self.opponentPoints += 1
		# update the points on the gui
		base.messenger.send("updatePoints", [self.points, self.opponentPoints])
		# it's our turn now
		self.setTurn(True)
		# select the next card
		self.nextCard()

	def nextCard(self):
		"""Select and show the next card in the deck. Also hide the
		previous active card. If no cards are left in the deck this
		function will send an endGame event"""
		# hide the current card
		self.currentCard.hideCard()
		self.currentCardOp.hideCard()
		# select the next card
		self.currentCardIdx -= 1
		if self.currentCardIdx < 0:
			# we have no cards left, so end the game
			base.messenger.send("endGame")
			return
		# set and draw the next card in the deck
		self.currentCard = self.playerDeck[self.currentCardIdx]
		self.currentCard.drawCard()
		self.currentCardOp = self.opponentDeck[self.currentCardIdx]
		self.currentCardOp.drawCard()

	def opponentIsBetter(self, isBetter):
		"""This Function is called to check if the opponent tell us if
		he is better or not. Dependent on the value we add points to our
		or the opponents point variable"""
		if isBetter == "True":
			# we loose that round and the opponent gets a point
			self.opponentPoints += 1
		else:
			# we won the round and get a point
			self.points += 1
		# update the points on the gui
		base.messenger.send("updatePoints", [self.points, self.opponentPoints])
		# move on and select the next card in the deck
		self.nextCard()

	def setPlayerDeck(self, deck):
		"""This function will set the player deck to the given deck.
		Then it will set all the models to be positioned on the table
		and draw the topmost card on the cards stack"""
		base.messenger.send("updateLoadingScreen", [95])
		# set the topmost z-position of the cardstack
		z = -1.18
		# now go through all cards in the deck and place them
		# appropriately in a stack
		for card in self.cardDeck:
			if card.id in deck:
				card.model.setPos(0.6, 6.8, z)
				card.model.reparentTo(render)
				self.playerDeck.append(card)
				z += 0.0262

		z = -1.18
		# get and place the opponents deck
		for card in self.cardDeck:
			if card not in self.playerDeck:
				card.model.setPos(-2.4, 11, z)
				card.model.reparentTo(render)
				self.opponentDeck.append(card)
				z += 0.0262
		base.messenger.send("updateLoadingScreen", [99])
		# select and draw the first card on the stack
		self.currentCard = self.playerDeck[self.currentCardIdx]
		self.currentCardOp = self.opponentDeck[self.currentCardIdx]
		base.messenger.send("updateLoadingScreen", [100])
		self.ls.hide()
		self.currentCard.drawCard()
		self.currentCardOp.drawOpCard()
		base.messenger.send("sendMessage", ["OPPONENT_IS_ONLINE"])

	def loadCards(self):
		"""Load all cards we have in the specified directory"""
		for (path, dirs, files) in os.walk("Data/cards"):
			for file in files:
				if file.endswith(".card"):
					f = os.path.join(path, file)
					self.cardDeck.append(Card(f))

	def endGame(self):
		"""This function is called at the end of the game. It will show
		the end message as well as hide the card selection GUI"""
		self.cardSelection.hide()
		EndGame(self.points > self.opponentPoints)

