from direct.gui.DirectGui import DirectButton
from direct.gui.DirectGui import DirectEntry
from direct.gui.DirectGui import DirectFrame
from direct.gui.DirectGui import DirectLabel
from direct.gui.DirectGui import DirectRadioButton
from direct.gui.DirectGui import DirectWaitBar
from direct.showbase.DirectObject import DirectObject
from panda3d.core import TextNode
from panda3d.core import Vec3

class LoadingScreen(DirectObject):
	"""This class represents a simple loading screen that should be
	shown each time a big amount of data will be loaded"""
	def __init__(self):
		# a fill panel so the player doesn't see how everything
		# gets loaded in the background
		self.frameMain = DirectFrame(
			# size of the frame
			frameSize = (base.a2dLeft, base.a2dRight,
						 base.a2dTop, base.a2dBottom),
			# tramsparent bg color
			frameColor = (0.16, 0.16, 0.16, 1),
			# set the order to 10000 so it will be above all other
			# elements on the screen
			sortOrder = 10000)

		# the text Loading... on top
		self.lblLoading = DirectLabel(
			scale = 0.25,
			pos = (0, 0, 0.5),
			frameColor = (0, 0, 0, 0),
			text = "Loading...",
			text_align = TextNode.ACenter,
			text_fg = (1,1,1,1))
		self.lblLoading.reparentTo(self.frameMain)

		# the waitbar on the bottom
		self.wbLoading = DirectWaitBar(
			text = "0%",
			text_fg = (1,1,1,1),
			value = 100,
			pos = (0, 0, -0.5),
			barColor = (0, 0, 1, 1))
		self.wbLoading.reparentTo(self.frameMain)

		# now show the main frame
		self.frameMain.show()
		# and render two frames so the loading screen
		# is realy shown on screen
		base.graphicsEngine.renderFrame()
		base.graphicsEngine.renderFrame()

		# accept an event to update the loading screen
		self.accept("updateLoadingScreen", self.setLoadingValue)

	def show(self):
		"""Hide the loading screen"""
		self.frameMain.show()

	def hide(self):
		"""Show the loading screen"""
		self.frameMain.hide()

	def setLoadingValue(self, value):
		"""Set the waitbar value to the given value, where
		value has to be a integer from 0 to 100"""
		# check if the value is good
		if value > 100: value = 100
		if value < 0: value = 0
		# set the value
		self.wbLoading["value"] = value
		self.wbLoading["text"] = "{0}%".format(value)
		base.graphicsEngine.renderFrame()
		base.graphicsEngine.renderFrame()

class Chat(DirectObject):
	"""This class represents the Chat and general info GUI part, located
	on the right side of the main screen."""
	def __init__(self):
		# maximum amount of lines of the textbox that will hold the send
		# and received messages
		self.textboxLines = 16

		# create a main frame as big as the window
		self.frameMain = DirectFrame(
			# set framesize the same size as the window
			frameSize = (.5, base.a2dRight,
						 base.a2dTop, base.a2dBottom),
			# position center
			pos = (0, 0, 0),
			# set tramsparent background color
			frameColor = (0.16, 0.16, 0.16, 1))

		# creates a lable that will show if an opponent is online or not
		self.lblOpponentOnPos = Vec3(0.53, 0, 0.8)
		self.lblOpponentOn = DirectLabel(
			pos = self.lblOpponentOnPos,
			text = "No opponent online",
			scale = 0.05,
			text_bg = (0,0,0,0),
			text_wordwrap = 10,
			text_align = TextNode.ALeft)
		self.lblOpponentOn.setTransparency(1)
		self.lblOpponentOn.reparentTo(self.frameMain)

		# the Textbox which stores all the messages we send
		# or recieved over the Network
		self.txtMessagesScale = 0.05
		self.txtMessagesPos = Vec3(0.54, 0, .6)
		self.txtMessages = DirectEntry(
			cursorKeys = True,
			pos = self.txtMessagesPos,
			scale = self.txtMessagesScale,
			width = 10,
			numLines = self.textboxLines)
		self.txtMessages.setTransparency(1)
		self.txtMessages.reparentTo(self.frameMain)

		# the Textbox where we write our Messages, which we
		# want to send over to the other users
		self.txtMessageScale = 0.05
		self.txtMessagePos = Vec3(0.54, 0, -0.3)
		self.txtMessage = DirectEntry(
			cursorKeys = True,
			pos = self.txtMessagePos,
			scale = self.txtMessageScale,
			width = 10,
			command = self.sendMsg)
		self.txtMessage.setTransparency(1)
		self.txtMessage.reparentTo(self.frameMain)

		# The button map for our send button
		#maps = loader.loadModel('Data/Button.egg')
		# create a send button for our entered messages
		self.btnSendPos = Vec3(0.75, 0, -.5)
		self.btnSendScale = 0.1
		self.btnSend = DirectButton(
			scale = self.btnSendScale,
			text = "Send",
            text_fg = (1,1,1,1),
			image = "Data/Button.png",
			frameSize = (-3,3, -.4,0.8),
            image_scale = (3, 0, 0.6),
            image_pos = (0, 0, 0.2),
			# position on the window
			pos = self.btnSendPos,
			# the event which is thrown on clickSound
			command = self.sendMsg,
			# sounds that should be played
			rolloverSound = None,
			clickSound = None)
		self.btnSend.setTransparency(1)
		self.btnSend.reparentTo(self.frameMain)

		# a label that will show the points we and the opponent have
		self.lblPointsPos = Vec3(0.63, 0, -0.8)
		self.lblPoints = DirectLabel(
			pos = self.lblPointsPos,
			text = "Points: 0\nEnemy Points: 0",
			scale = 0.05,
			text_bg = (0,0,0,0),
			text_wordwrap = 10,
			text_align = TextNode.ALeft)
		self.lblPoints.setTransparency(1)
		self.lblPoints.reparentTo(self.frameMain)

		# catch window resizes and recalculate the aspectration
		self.accept('window-event', self.recalcAspectRatio)
		self.recalcAspectRatio(base.win)

		# accept all update events
		self.accept("gotMessage", self.setText)
		self.accept("updatePoints", self.setPoints)

	def sendMsg(self, msg=None):
		"""Send the message we have in the textbox and clear the textbox
		after sending"""
		msg = self.txtMessage.get() if msg == None else msg
		base.messenger.send("sendMessage", [msg])
		self.txtMessage.enterText("")

	def setText(self, messageText):
		"""Function which will write the given text in the
		textbox, where we store all send and recieved messages.
		This Function will only write the last 11 messages and
		cut of the erlier messages"""
		# get all messages from the textbox
		parts = self.txtMessages.get().split("\n")
		if len(parts) >= self.textboxLines:
			cutParts = ""
			# as the textbox can only hold the given amount of lines cut
			# out the first entry
			for i in range(1,len(parts)):
				cutParts += parts[i] + "\n"
			self.txtMessages.enterText(cutParts + messageText)
		else:
			self.txtMessages.enterText(self.txtMessages.get() + "\n" + messageText)

	def setPoints(self, points, opponentPoints):
		"""Set the points displayed on the gui to the given properties"""
		self.lblPoints["text"] = "Points: {}\nEnemy Points: {}".format(
			points, opponentPoints)
		self.lblPoints.setText()

	def opponentOnline(self, isOnline):
		"""Set the text wether an opponent is online or not"""
		if isOnline:
			self.lblOpponentOn["text"] = "An opponent is ON!"
		else:
			self.lblOpponentOn["text"] = "No opponent online"
		self.lblPoints.setText()

	def show(self):
		"""Show the GUI"""
		self.frameMain.show()

	def hide(self):
		"""Hide the GUI"""
		self.frameMain.hide()

	def recalcAspectRatio(self, window):
		"""get the new aspect ratio to resize the mainframe"""
		# calculate new aspec tratio
		wp = window.getProperties()
		aspX = 1.0
		aspY = 1.0
		wpXSize = wp.getXSize()
		wpYSize = wp.getYSize()
		if wpXSize > wpYSize:
			aspX = wpXSize / float(wpYSize)
		else:
			aspY = wpYSize / float(wpXSize)

		# set new positions/size/etc for all the gui elements
		# in this class
		self.frameMain["frameSize"] = (
			.5 * aspX, base.a2dRight,
			base.a2dTop, base.a2dBottom)
		self.lblOpponentOn.setPos(
			self.lblOpponentOnPos.getX() * aspX,
			self.lblOpponentOnPos.getY(),
			self.lblOpponentOnPos.getZ() * aspY)
		self.txtMessages.setPos(
			self.txtMessagesPos.getX() * aspX,
			self.txtMessagesPos.getY(),
			self.txtMessagesPos.getZ() * aspY)
		self.txtMessage.setPos(
			self.txtMessagePos.getX() * aspX,
			self.txtMessagePos.getY(),
			self.txtMessagePos.getZ() * aspY)
		self.btnSend.setPos(
			self.btnSendPos.getX() * aspX,
			self.btnSendPos.getY(),
			self.btnSendPos.getZ() * aspY)
		self.lblPoints.setPos(
			self.lblPointsPos.getX() * aspX,
			self.lblPointsPos.getY(),
			self.lblPointsPos.getZ() * aspY)

class CardSelection():
	"""This GUI displays a selection for the card values"""
	def __init__(self, names):
		"""Create radiobuttons for each name in the names list and a button
		to confirm the selection the user made. All buttons are placed in a
		column in the center of the main part of the screen"""
		self.selected = [names[0]]
		self.buttons = []
		pos = Vec3(-0.4, 0, -0.5)
		idx = 0

		# create the radio buttons
		for name in names:
			self.buttons.append(DirectRadioButton(
				text = name,
				variable = self.selected,
				value = [name],
				scale = 0.05,
				pos = pos,
				text_align = TextNode.ALeft))
			pos.setZ(pos.getZ() - 0.1)
			idx+=1

		# combine the radio buttons in one group
		for button in self.buttons:
			button.setOthers(self.buttons)
		self.buttons[0]["indicatorValue"] = 1

		# create a select button
		pos.setZ(pos.getZ() - 0.05)
		pos.setX(pos.getX() - 0.05)
		self.btnSelect = DirectButton(
			scale = 0.1,
			text = "Select",
			# position on the window
			pos = pos,
			# sounds that should be played
			rolloverSound = None,
			clickSound = None,
			text_align = TextNode.ALeft,
			command = self.sendSelection)
		self.btnSelect.setTransparency(1)

	def sendSelection(self):
		"""This function will send the selected option and hide the
		selection-gui"""
		base.messenger.send("guiSelected", self.selected)
		self.hide()

	def show(self):
		"""Show the GUI"""
		for button in self.buttons:
			button.show()
		self.btnSelect.show()

	def hide(self):
		"""Hide the GUI"""
		for button in self.buttons:
			button.hide()
		self.btnSelect.hide()

class EndGame():
	"""This class represents the GUI which will be shown at the end of
	the game"""
	def __init__(self, won):
		# create a quit button
		self.btnQuitPos = Vec3(-0.25, 0, -0.5)
		self.btnQuitScale = 0.2
		self.btnQuit = DirectButton(
			scale = self.btnQuitScale,
			text = "Quit",
			# position on the window
			pos = self.btnQuitPos,
			# the event which is thrown on clickSound
			command = self.quit,
			# sounds that should be played
			rolloverSound = None,
			clickSound = None)
		self.btnQuit.setTransparency(1)

		# set the text dependent on the given property
		if won:
			txt = "you won"
		else:
			txt = "you loose"

		# create a label which tells the player if he has won or loose
		# the game
		self.lblWinPos = Vec3(-0.25, 0, 0.3)
		self.lblWin = DirectLabel(
			pos = self.lblWinPos,
			text = txt,
			scale = 0.35,
			text_bg = (0,0,0,0),
			text_align = TextNode.ACenter)
		self.lblWin.setTransparency(1)

	def quit(self):
		"""Send a message to quit the game"""
		self.hide()
		base.messenger.send("quit")

	def show(self):
		"""Show the GUI"""
		self.btnQuit.show()
		self.lblWin.show()

	def hide(self):
		"""Hide the GUI"""
		self.btnQuit.hide()
		self.lblWin.hide()
